/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useState } from 'react';
import {
  AppBar,
  Toolbar,
  CssBaseline,
  Typography,
  MenuItem,
  ListItemText,
  Menu,
  IconButton,
  Button,
} from '@mui/material';
import { AccountCircle } from '@mui/icons-material';
import logo from './logogp.png';
import logo_white from './logogpwhite.png';
import AddDatabaseForm from './AddDatabaseForm';
import { useTheme } from '@mui/material/styles';
import styles from './navbar.module.scss';
import {
  AddDatabase,
  AddDatabaseRequest,
  GetAllDatabases,
  RequestSchema,
  useAuthorization,
} from '@graphpolaris/shared/lib/data-access';
import {
  updateCurrentDatabase,
  updateDatabaseList,
} from '@graphpolaris/shared/lib/data-access/store/sessionSlice';

/** NavbarComponentProps is an interface containing the NavbarViewModel. */
export interface NavbarComponentProps {
  // changeColourPalette: () => void; FIXME move to redux
}

/** NavbarComponentState is an interface containing the type of visualizations. */
export interface NavbarComponentState {
  isAuthorized: boolean;
  clientID: string;
  sessionID: string;
  databases: string[];
  currentDatabase: string;

  // The anchor for rendering the user data (clientID, sessionID, add database, etc) menu
  userMenuAnchor?: Element;
  // The anchor for rendering the menu for selecting a database to use
  selectDatabaseMenuAnchor?: Element;
  // Determines if the addDatabaseForm will be shown
  showAddDatabaseForm: boolean;
}

/** NavbarComponent is the View implementation for Navbar */
export const Navbar = (props: NavbarComponentProps) => {
  const theme = useTheme();
  const authorization = useAuthorization();

  // const { navbarViewModel, currentColours } = props;
  // this.navbarViewModel = navbarViewModel;

  const [state, setState] = useState<NavbarComponentState>({
    isAuthorized: false,
    clientID: authorization.userAuthorized ? authorization.userId || '' : '',
    sessionID: authorization.sessionId || '',
    databases: [],
    currentDatabase: '',

    userMenuAnchor: undefined,
    selectDatabaseMenuAnchor: undefined,
    showAddDatabaseForm: false,
  });

  /** Closes the user menu. Also closes all nested menu's. */
  function closeUserMenu(): void {
    // If a nested window is open, close the main user menu a bit later
    if (state.selectDatabaseMenuAnchor != undefined) {
      setState({ ...state, selectDatabaseMenuAnchor: undefined });
      setTimeout(() => setState({ ...state, userMenuAnchor: undefined }), 100);
    } else setState({ ...state, userMenuAnchor: undefined });
  }

  function changeColourPalette() {
    // TODO
  }

  /**
   * Called when the user clicks on the 'submit' button of the add database form.
   */
  function onAddDatabaseFormSubmit(
    request: AddDatabaseRequest
  ): Promise<void | Response> {
    return AddDatabase(request).then(() =>
      GetAllDatabases().then((databases) => {
        updateDatabaseList(databases);
        updateCurrentDatabase(request.name);
        // When the database changes, request the new schema
        console.log('databases ' + databases);
        RequestSchema(request.name);
      })
    );
  }

  const currentLogo = theme.palette.custom.logo == 'white' ? logo_white : logo;

  return (
    <div className={styles.root}>
      <CssBaseline />
      <AppBar
        title="GraphPolaris"
        style={{
          zIndex: 1250,
          backgroundColor: theme.palette.custom.background,
        }}
        position="fixed"
        className={styles.appBar}
      >
        <Toolbar style={{ marginLeft: -5 }}>
          <a href="https://graphpolaris.com/" className={styles.logo}>
            <img src={currentLogo} className={styles.logo} />
          </a>
          <div className={styles.menubox}>
            <Button
              className={styles.menuText}
              style={{ color: theme.palette.custom.menuText }}
              onClick={changeColourPalette}
            >
              Change Palette
            </Button>
            <Button
              href="https://graphpolaris.com/"
              className={styles.menuText}
              style={{ color: theme.palette.custom.menuText }}
            >
              Home
            </Button>
            <Button
              href="https://graphpolaris.com/index.php/products/"
              className={styles.menuText}
              style={{ color: theme.palette.custom.menuText }}
            >
              Products
            </Button>
            <Button
              href="https://graphpolaris.com#usecases"
              className={styles.menuText}
              style={{ color: theme.palette.custom.menuText }}
            >
              Use Cases
            </Button>
            <Button
              href="https://graphpolaris.com#earlyadoption"
              className={styles.menuText}
              style={{ color: theme.palette.custom.menuText }}
            >
              Contact
            </Button>
            {state.isAuthorized ? (
              <div>
                <IconButton
                  color="inherit"
                  onClick={(event) =>
                    setState({
                      ...state,
                      userMenuAnchor: event.currentTarget,
                    })
                  }
                >
                  <AccountCircle htmlColor={theme.palette.custom.menuText} />
                </IconButton>
                <Menu
                  id="user-menus"
                  anchorOrigin={{ vertical: 'bottom', horizontal: 'left' }}
                  transformOrigin={{ vertical: 'top', horizontal: 'left' }}
                  anchorEl={state.userMenuAnchor}
                  keepMounted
                  open={Boolean(state.userMenuAnchor)}
                  onClose={() => closeUserMenu()}
                >
                  <MenuItem>
                    <ListItemText primary={'clientID: ' + state.clientID} />
                  </MenuItem>
                  <MenuItem>
                    <ListItemText primary={'sessionID: ' + state.sessionID} />
                  </MenuItem>
                  {/* <MenuItem
                    onClick={() =>
                      navbarViewModel.handleLoginButtonClicked('google')
                    }
                  >
                    <ListItemText primary={'request new'} />
                  </MenuItem> */}
                  {/*<MenuItem onClick={() => LocalStorage.instance().Reset()}>*/}
                  {/*  <ListItemText primary={'Empty localstorage'} />*/}
                  {/*</MenuItem>*/}
                  <MenuItem
                    onClick={() =>
                      setState({
                        ...state,
                        userMenuAnchor: undefined,
                        showAddDatabaseForm: true,
                      })
                    }
                  >
                    <ListItemText primary={'Add database'} />
                  </MenuItem>
                  <MenuItem
                    onClick={(event) =>
                      setState({
                        ...state,
                        selectDatabaseMenuAnchor: event.currentTarget,
                      })
                    }
                  >
                    <ListItemText primary={'Change database'} />
                  </MenuItem>
                  <Menu
                    id="databases-menus"
                    anchorOrigin={{ vertical: 'top', horizontal: 'left' }}
                    transformOrigin={{ vertical: 'top', horizontal: 'right' }}
                    anchorEl={state.selectDatabaseMenuAnchor}
                    keepMounted
                    disableAutoFocusItem
                    open={Boolean(state.selectDatabaseMenuAnchor)}
                    onClose={() => closeUserMenu()}
                  >
                    {state.databases.length > 0 ? (
                      state.databases.map((database) => (
                        <MenuItem
                          key={database}
                          selected={database == state.currentDatabase}
                          onClick={() => {
                            if (state.currentDatabase != database) {
                              updateCurrentDatabase(database);
                              closeUserMenu();
                            }
                          }}
                        >
                          <ListItemText primary={database} />
                        </MenuItem>
                      ))
                    ) : (
                      <MenuItem key="placeholder" value="" disabled>
                        no databases connected
                      </MenuItem>
                    )}
                  </Menu>
                </Menu>
              </div>
            ) : (
              <div>
                <Button
                  className={styles.loginButton}
                  onClick={(event) =>
                    setState({
                      ...state,
                      userMenuAnchor: event.currentTarget,
                    })
                  }
                >
                  <span
                    className={styles.loginButtonText}
                    style={{
                      color: theme.palette.custom.menuText,
                    }}
                  >
                    Login
                  </span>
                </Button>
                {/* <Menu
                  id="user-menus"
                  getContentAnchorEl={null}
                  anchorOrigin={{ vertical: 'bottom', horizontal: 'left' }}
                  transformOrigin={{ vertical: 'top', horizontal: 'left' }}
                  anchorEl={state.userMenuAnchor}
                  keepMounted
                  open={Boolean(state.userMenuAnchor)}
                  onClose={() => closeUserMenu()}
                >
                  <MenuItem
                    onClick={() =>
                      navbarViewModel.handleLoginButtonClicked('google')
                    }
                  >
                    <ListItemText primary={'Login with Google'} />
                  </MenuItem>
                  <MenuItem
                    onClick={() =>
                      navbarViewModel.handleLoginButtonClicked('github')
                    }
                  >
                    <ListItemText primary={'Login with GitHub'} />
                  </MenuItem>
                  <MenuItem
                    onClick={() =>
                      navbarViewModel.handleLoginButtonClicked('free')
                    }
                  >
                    <ListItemText primary={'Login with free (debug)'} />
                  </MenuItem>
                </Menu> */}
              </div>
            )}
          </div>
        </Toolbar>
      </AppBar>
      <AddDatabaseForm
        open={state.showAddDatabaseForm}
        onClose={() => setState({ ...state, showAddDatabaseForm: false })}
        onSubmit={(...params) => {
          onAddDatabaseFormSubmit(...params);
          setState({ ...state, showAddDatabaseForm: false });
        }}
      />
    </div>
  );
};
